package com.bocloud.sms.booter.controller;

import com.alibaba.fastjson.JSONArray;
import com.bocloud.sms.interfaces.SystemConfigService;
import com.bocloud.sms.model.ConfigModel;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Result;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author lyy
 * @Version 1.0
 * @since 2017年8月18日
 */
@RestController
@RequestMapping("/v1")
@Tag(name = "系统配置管理")
public class SystemConfigController {

    private final SystemConfigService systemConfigService;

    public SystemConfigController(SystemConfigService systemConfigService) {
        this.systemConfigService = systemConfigService;
    }

    /**
     * 更新系统配置
     *
     * @param requestContext 上下文
     * @param request        浏览器请求
     * @return 更新结果
     */
    @Operation(summary = "更新系统配置")
    @PutMapping("/system-configs")
    public Result modify(@Value(Common.REQ_CONTEXT) RequestContext requestContext, HttpServletRequest request) {
        List<ConfigModel> configs = JSONArray.parseArray(request.getParameter("configs"), ConfigModel.class);
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        return systemConfigService.batchUpdate(configs, fileMap, requestContext);
    }


    /**
     * 同步Ldap
     *
     * @return 同步结果
     */
    @Operation(summary = "同步Ldap")
    @PostMapping("/configs/syncLdap")
    public Result syncLdap() {
        return systemConfigService.syncLdap();
    }

    /**
     * 查询系统配置
     *
     * @param category
     * @param codes
     * @return
     */
    @Operation(summary = "批量查询系统配置")
    @GetMapping("/configs")
    public GeneralResult<Map<String, String>> getByCodes(@RequestParam(value = "category", required = false) String category, @RequestParam(value = "codes", required = false) ArrayList<String> codes) {
        return systemConfigService.listConfigs(category, codes);
    }


    /**
     * 查询系统配置类型树
     *
     * @param category
     * @return
     */
    @Operation(summary = "查询系统配置类型树")
    @GetMapping("/configs/tree")
    public GeneralResult<JSONArray> listCategoryTree(@RequestParam(value = "category") String category) {
        return systemConfigService.listCategoryTree(category);
    }


    /**
     * 系统配置连接测试
     *
     * @param category
     * @return
     */
    @Operation(summary = "系统配置连接测试")
    @GetMapping("/configs/test")
    public Result configTest(@RequestParam(value = "category") String category) {
        return systemConfigService.configTest(category);
    }
}
