package com.bocloud.sms.service.internel;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BoCloudService;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.bocloud.sms.entity.TenantAccount;
import com.bocloud.sms.model.ServiceBean;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * scp服务内部访问
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2018/12/18
 */
@Component
public class CosInternelService implements InitializingBean {

    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;
    private static final Service SERVICE = Service.create(BoCloudService.COS.getName());
    private final Map<String, Object> headers = new HashMap<>(4);

    public CosInternelService(ServiceFactory serviceFactory, CurrentService currentService) {
        this.serviceFactory = serviceFactory;
        this.currentService = currentService;
    }

    /**
     * 删除租户授权的服务
     *
     * @param tenantId 租户id
     * @return
     */
    public GeneralResult deleteByTenant(Long tenantId) {
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/services/tenants/" + tenantId,
                HttpMethod.DELETE, headers, null);
        return remoteService.invoke();
    }

    /**
     * 租户授权服务
     *
     * @param tenantId 租户id
     * @param services 服务集合
     * @return
     */
    public GeneralResult grant(Long tenantId, List<ServiceBean> services) {
        Map<String, Object> paramMap = new HashMap<>(1);
        JSONObject object = new JSONObject();
        object.put("tenantId", tenantId);
        object.put("services", services);
        paramMap.put(Common.PARAMS, object.toJSONString());
        RemoteService remoteService = serviceFactory.build(SERVICE,
                "/v1/services/tenants/" + tenantId + "/grant", HttpMethod.POST, headers, paramMap);
        return remoteService.invoke();
    }

    @SuppressWarnings("unused")
    public GeneralResult panelDefaultSetting(String apiKey, Long tenantId) {
        headers.put("apiKey", apiKey);
        JSONObject object = new JSONObject();
        object.put("tenantId", tenantId);
        Map<String, Object> params = MapTools.simpleMap(Common.PARAMS, object.toJSONString());
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/portal/panels/tenants",
                HttpMethod.POST, headers, params);
        return remoteService.invoke();
    }

    /**
     * 删除租户首页配置
     *
     * @param apiKey   请求api key
     * @param tenantId 租户id
     * @return
     */
    public GeneralResult deleteByTenantId(String apiKey, Long tenantId) {
        headers.put("apiKey", apiKey);
        JSONObject object = new JSONObject();
        object.put("tenantId", tenantId);
        Map<String, Object> params = MapTools.simpleMap(Common.PARAMS, object.toJSONString());
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/portal/panels/tenants",
                HttpMethod.DELETE, headers, params);
        return remoteService.invoke();
    }

    /**
     * 查询租户优惠券数量
     *
     * @param tenantId    租户id
     * @param promotionId 优惠活动id
     * @return
     */
    public Integer tenantCoupons(Long tenantId, Long promotionId) {
        Map<String, Object> params = MapTools.simpleMap("tenantId", tenantId);
        params.put("promotionId", promotionId);
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/coupons/tenants", HttpMethod.GET, headers,
                params);
        GeneralResult result = remoteService.invoke();
        if (result.isFailed()) {
            return 0;
        }
        return Integer.valueOf(String.valueOf(result.getData()));
    }

    /**
     * 服务配额校验
     *
     * @param id      id
     * @param catalog 账号类型
     * @return
     */
    public GeneralResult serviceQuotaCheck(Long id, String catalog) {
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("catalog", catalog);
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/quotas/check", HttpMethod.GET, headers,
                params);
        return remoteService.invoke();
    }

    /**
     * 查询授信账户详情
     *
     * @param tenantId
     * @param apiKey
     * @return
     */
    public GeneralResult tenantAccountDetail(Long tenantId, String apiKey) {
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/tenants/" + tenantId + "/account", HttpMethod.GET, headers, null);
        return remoteService.invoke();
    }

    /**
     * 创建授信账户
     *
     * @param tenantAccount
     * @param apiKey
     * @return
     */
    public GeneralResult tenantAccountAssign(TenantAccount tenantAccount, String apiKey) {
        Map<String, Object> params = MapTools.simpleMap(Common.PARAMS, JSONObject.toJSONString(tenantAccount));
        headers.put(Common.API_KEY, apiKey);
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/tenants/account", HttpMethod.POST, headers,
                params);
        return remoteService.invoke();
    }

    /**
     * 删除授信账户
     *
     * @param tenantId
     * @param apiKey
     * @return
     */
    public GeneralResult deleteTenantAccount(Long tenantId, String apiKey) {
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/tenants/" + tenantId + "/account", HttpMethod.DELETE, headers, null);
        return remoteService.invoke();
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());
    }
}