package com.bocloud.sms.service.internel;

import com.bocloud.cmp.boot.model.BocloudToken;
import com.megatron.common.model.GeneralResult;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.model.RemoteService;
import com.megatron.framework.http.model.ServiceParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * @author zhangyf
 */
@Slf4j
@Component
public class PluginInsideService {

    private final static String SCHEMA = "http://";

    private final CurrentService currentService;

    public PluginInsideService(CurrentService currentService) {
        this.currentService = currentService;
    }

    /**
     * 安装插件
     *
     * @param service    服务
     * @param instance   实例地址
     * @param pluginPath 插件路径
     */
    public void install(String service, String instance, String pluginPath) {
        Map<String, Object> headers = new HashMap<>(4);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        String url = SCHEMA + instance + "/v1/plugins/install?pluginPath=" + pluginPath;
        ServiceParam param = new ServiceParam(null, headers);
        RemoteService remoteService = new RemoteService(Service.create(service), param, true, url, HttpMethod.PUT);
        GeneralResult<?> result = remoteService.invoke();
        if (result.isFailed()) {
            log.error("插件安装失败：" + result.getMessage());
        }
    }

    /**
     * 更新插件
     *
     * @param service    服务
     * @param instance   实例地址
     * @param pluginPath 插件路径
     */
    public void upgrade(String service, String instance, String pluginId, String pluginPath) {
        Map<String, Object> headers = new HashMap<>(4);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        String url = SCHEMA + instance + "/v1/plugins/" + pluginId + "/upgrade?pluginPath=" + pluginPath;
        ServiceParam param = new ServiceParam(null, headers);
        RemoteService remoteService = new RemoteService(Service.create(service), param, true, url, HttpMethod.PUT);
        GeneralResult<?> result = remoteService.invoke();
        if (result.isFailed()) {
            log.error("插件更新失败：" + result.getMessage());
        }
    }

    /**
     * 启动插件
     *
     * @param service  服务
     * @param instance 实例地址
     * @param pluginId 插件编码
     */
    public void start(String service, String instance, String pluginId) {
        Map<String, Object> headers = new HashMap<>(4);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        String url = SCHEMA + instance + "/v1/plugins/" + pluginId + "/start";
        ServiceParam param = new ServiceParam(null, headers);
        RemoteService remoteService = new RemoteService(Service.create(service), param, true, url, HttpMethod.PUT);
        GeneralResult<?> result = remoteService.invoke();
        if (result.isFailed()) {
            log.error("插件启动失败：" + result.getMessage());
        }
    }

    /**
     * 停止插件
     *
     * @param service  服务
     * @param instance 实例地址
     * @param pluginId 插件编码
     */
    public void stop(String service, String instance, String pluginId) {
        Map<String, Object> headers = new HashMap<>(4);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        String url = SCHEMA + instance + "/v1/plugins/" + pluginId + "/stop";
        ServiceParam param = new ServiceParam(null, headers);
        RemoteService remoteService = new RemoteService(Service.create(service), param, true, url, HttpMethod.PUT);
        GeneralResult<?> result = remoteService.invoke();
        if (result.isFailed()) {
            log.error("插件停止失败：" + result.getMessage());
        }
    }

    /**
     * 卸载插件
     *
     * @param service  服务
     * @param instance 实例地址
     * @param pluginId 插件编码
     */
    public void uninstall(String service, String instance, String pluginId) {
        Map<String, Object> headers = new HashMap<>(4);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        String url = SCHEMA + instance + "/v1/plugins/" + pluginId + "/uninstall";
        ServiceParam param = new ServiceParam(null, headers);
        RemoteService remoteService = new RemoteService(Service.create(service), param, true, url, HttpMethod.PUT);
        GeneralResult<?> result = remoteService.invoke();
        if (result.isFailed()) {
            log.error("插件卸载失败：" + result.getMessage());
        }
    }

}
